"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const url = require("url");
const defaultHeaders = {
    'User-Agent': 'graphql-url-to-typescript/0.0 (+https://github.com/alexisfontaine/typed-graph)',
    'Content-Type': 'application/json; charset=utf8',
};
function ask(entrypoint, query, options = {}) {
    if (!entrypoint)
        return Promise.reject(new Error('No URL provided'));
    const body = JSON.stringify({ query });
    const headers = Object.assign({}, defaultHeaders, { 'Content-Length': Buffer.byteLength(body) });
    return new Promise((resolve, reject) => {
        const formattedUrl = url.parse(entrypoint);
        const request = (formattedUrl.protocol === 'http' ? http.request : https.request)({
            method: 'POST',
            hostname: formattedUrl.hostname,
            path: formattedUrl.path,
            headers: options.headers ? Object.assign(headers, options.headers) : headers
        }, response => {
            let result = '';
            let error;
            response
                .setEncoding('utf8')
                .on('data', chunk => result += chunk)
                .on('end', () => {
                if (response.headers['content-type'].includes('application/json')) {
                    try {
                        result = JSON.parse(result);
                    }
                    catch (_) {
                        error = {
                            message: 'Unable to parse the response body',
                            value: result
                        };
                    }
                }
                if (response.statusCode && response.statusCode < 400)
                    return resolve(result);
                return reject({
                    error: {
                        message: `Failed request to "${formattedUrl.href}"`,
                        statusCode: response.statusCode,
                        statusMessage: response.statusMessage,
                        requestHeaders: request.getHeaders(),
                        responseHeaders: response.headers,
                        responseBody: error || result
                    }
                });
            });
        });
        request.write(body);
        request.end();
    });
}
exports.ask = ask;
exports.default = ask;
